function dy2 = Diffusion_Solver(dy,dy2,G1,D,e,dx)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                        D I F F U S I O N                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                                
% If you provide it with the grid locations of the four boundaries for    %
% each zone as G it will go through and solve the central difference      %
% diffusion problem and output it as dy2 for the ODE solver. This         % 
% solution is taken from Raf Roelants Thesis: Mathematical determination  %
% of reaction networks from transient kinetic experiments (2011).         %
% Created by Christian Reece during the coronavirus pandemic              %
% March 18th 2020                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First point of Z1 :       GX(1)
dy2(G1(1),1) = D(1,1)/(dx*dx)/e(1)*(dy(G1(1)+1)-dy(G1(1)));

% Internal points of Z1 :   GX(1)+1:GX(2)-2
dy2( (G1(1)+1:G1(2)-2) ,1) = D(1,1)/(dx*dx)/e(1)*...
    (dy( (G1(1)+1:G1(2)-2) +1 ) - 2*dy( (G1(1)+1:G1(2)-2) ) + dy( (G1(1)+1:G1(2)-2) -1 ));

% Final point of Z1 :       GX(2)-1
dy2( (G1(2)-1) ,1) = (-D(1,1)/(dx*dx)*(dy( (G1(2)-1) )-dy( (G1(2)-1) -1))...
    + 2/dx/(dx/D(1,1) + dx/D(1,2))...
    *(dy( (G1(2)-1) +1)-dy( (G1(2)-1) )))/e(1);

% First point of Z2 :       GX(2)
dy2( (G1(2)) ,1) = (-2/dx/(dx/D(1,1) + dx/D(1,2))...
    *(dy( (G1(2)) )-dy( (G1(2)) -1 ))...
    + D(1,2)/(dx*dx)*(dy( (G1(2)) +1 )-dy( (G1(2)) )))/e(2);

% Internal points of Z2 :   GX(2)+1:GX(3)-2
dy2( (G1(2)+1:G1(3)-2) ,1) = D(1,2)/(dx*dx)/e(2)*...
    (dy( (G1(2)+1:G1(3)-2) +1) - 2*dy( (G1(2)+1:G1(3)-2) ) + dy( (G1(2)+1:G1(3)-2) -1));

% Final point of Z2 :       GX(3)-1
dy2( (G1(3)-1) ,1) = (-D(1,2)/(dx*dx)*(dy( (G1(3)-1) )-dy( (G1(3)-1) -1))...
    +2/dx/(dx/D(1,2) + dx/D(1,3))...
    *(dy( (G1(3)-1) +1)-dy( (G1(3)-1) )))/e(2);

% First point of Z3 :       GX(3)
dy2( G1(3) ,1) = (-2/dx/(dx/D(1,2) + dx/D(1,3))...
    *(dy( G1(3) )-dy( G1(3) -1))...
    + D(1,3)/(dx*dx)*(dy( G1(3) +1)-dy( G1(3) )))/e(3);

% Internal points in Z3 :   GX(3)+1:GX(4)-1
dy2( (G1(3)+1:G1(4)-1) ,1) = D(1,3)/(dx*dx)/e(3)*...
    (dy( (G1(3)+1:G1(4)-1) +1) - 2*dy( (G1(3)+1:G1(4)-1) ) + dy( (G1(3)+1:G1(4)-1) -1));

% Final point in Z3 :       GX(4)
dy2( G1(4) ,1) = D(1,3)/(dx*dx)/e(3)*...
    (-2*dy( G1(4) )+dy( G1(4) -1));
end